<?php
namespace Opencart\Catalog\Controller\Extension\Infrapay\Payment;
class Infrapay extends \Opencart\System\Engine\Controller {
	public function index(): string {
		$this->load->language('extension/infrapay/payment/infrapay');
		$data['language'] = $this->config->get('config_language');
		return $this->load->view('extension/infrapay/payment/infrapay', $data);
	}

	public function confirm(): void {
		$json = [];
		$this->load->language('extension/infrapay/payment/infrapay');
        $session = $this->session->data;
        $order_id = isset($session['order_id']) ? $session['order_id'] : false;
		if (!$order_id) {
			$json['error'] = $this->language->get('error_order');
		}
		if (!isset(  $session['payment_method']) ||   $session['payment_method'] != 'infrapay') {
			$json['error'] = $this->language->get('error_payment_method');
		}
		if (!$json) {
			$this->load->model('checkout/order');
            $totals = [];
            $taxes = $this->cart->getTaxes();
            $total = 0;
            $this->load->model('checkout/cart');
            ($this->model_checkout_cart->getTotals)($totals, $taxes, $total);
            $order_data = [];
			$order_data['invoice_prefix'] = $this->config->get('config_invoice_prefix');
			$order_data['store_id'] = $this->config->get('config_store_id');
			$order_data['store_name'] = $this->config->get('config_name');
			$order_data['store_url'] = $this->config->get('config_url');
			$order_data['customer_id'] =   $session['customer']['customer_id'];
			$order_data['customer_group_id'] =   $session['customer']['customer_group_id'];
			$order_data['firstname'] =   $session['customer']['firstname'];
			$order_data['lastname'] =   $session['customer']['lastname'];
			$order_data['email'] =   $session['customer']['email'];
			$order_data['telephone'] =   $session['customer']['telephone'];
			$order_data['custom_field'] =   $session['customer']['custom_field'];
			if ($this->config->get('config_checkout_address')) {
				$order_data['payment_firstname'] =   $session['payment_address']['firstname'];
				$order_data['payment_lastname'] =   $session['payment_address']['lastname'];
				$order_data['payment_company'] =   $session['payment_address']['company'];
				$order_data['payment_address_1'] =   $session['payment_address']['address_1'];
				$order_data['payment_address_2'] =   $session['payment_address']['address_2'];
				$order_data['payment_city'] =   $session['payment_address']['city'];
				$order_data['payment_postcode'] =   $session['payment_address']['postcode'];
				$order_data['payment_zone'] =   $session['payment_address']['zone'];
				$order_data['payment_zone_id'] =   $session['payment_address']['zone_id'];
				$order_data['payment_country'] =   $session['payment_address']['country'];
				$order_data['payment_country_id'] =   $session['payment_address']['country_id'];
				$order_data['payment_address_format'] =   $session['payment_address']['address_format'];
				$order_data['payment_custom_field'] = isset(  $session['payment_address']['custom_field']) ?   $session['payment_address']['custom_field'] : [];
			} else {
				$order_data['payment_firstname'] = '';
				$order_data['payment_lastname'] = '';
				$order_data['payment_company'] = '';
				$order_data['payment_address_1'] = '';
				$order_data['payment_address_2'] = '';
				$order_data['payment_city'] = '';
				$order_data['payment_postcode'] = '';
				$order_data['payment_zone'] = '';
				$order_data['payment_zone_id'] = 0;
				$order_data['payment_country'] = '';
				$order_data['payment_country_id'] = 0;
				$order_data['payment_address_format'] = '';
				$order_data['payment_custom_field'] = [];
			}
			$payment_method_info =   $session['payment_methods'][  $session['payment_method']];
			$order_data['payment_method'] = $payment_method_info['title'];
			$order_data['payment_code'] = $payment_method_info['code'];
			if ($this->cart->hasShipping()) {
				$order_data['shipping_firstname'] =   $session['shipping_address']['firstname'];
				$order_data['shipping_lastname'] =   $session['shipping_address']['lastname'];
				$order_data['shipping_company'] =   $session['shipping_address']['company'];
				$order_data['shipping_address_1'] =   $session['shipping_address']['address_1'];
				$order_data['shipping_address_2'] =   $session['shipping_address']['address_2'];
				$order_data['shipping_city'] =   $session['shipping_address']['city'];
				$order_data['shipping_postcode'] =   $session['shipping_address']['postcode'];
				$order_data['shipping_zone'] =   $session['shipping_address']['zone'];
				$order_data['shipping_zone_id'] =   $session['shipping_address']['zone_id'];
				$order_data['shipping_country'] =   $session['shipping_address']['country'];
				$order_data['shipping_country_id'] =   $session['shipping_address']['country_id'];
				$order_data['shipping_address_format'] =   $session['shipping_address']['address_format'];
				$order_data['shipping_custom_field'] = isset(  $session['shipping_address']['custom_field']) ?   $session['shipping_address']['custom_field'] : [];
			} else {
				$order_data['shipping_firstname'] = '';
				$order_data['shipping_lastname'] = '';
				$order_data['shipping_company'] = '';
				$order_data['shipping_address_1'] = '';
				$order_data['shipping_address_2'] = '';
				$order_data['shipping_city'] = '';
				$order_data['shipping_postcode'] = '';
				$order_data['shipping_zone'] = '';
				$order_data['shipping_zone_id'] = 0;
				$order_data['shipping_country'] = '';
				$order_data['shipping_country_id'] = 0;
				$order_data['shipping_address_format'] = '';
				$order_data['shipping_custom_field'] = [];
			}
			if (isset(  $session['shipping_method'])) {
				$shipping = explode('.', $session['shipping_method']);
				$shipping_method_info = $session['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];
				$order_data['shipping_method'] = $shipping_method_info['title'];
				$order_data['shipping_code'] = $shipping_method_info['code'];
			} else {
				$order_data['shipping_method'] = '';
				$order_data['shipping_code'] = '';
			}
			$this->load->model('checkout/cart');
			$order_data['products'] = [];
			$products = $this->model_checkout_cart->getProducts();
			foreach ($products as $product) {
				$option_data = [];

				foreach ($product['option'] as $option) {
					$option_data[] = [
						'product_option_id'       => $option['product_option_id'],
						'product_option_value_id' => $option['product_option_value_id'],
						'option_id'               => $option['option_id'],
						'option_value_id'         => $option['option_value_id'],
						'name'                    => $option['name'],
						'value'                   => $option['value'],
						'type'                    => $option['type']
					];
				}
                $order_data['products'][] = [
                        'product_id'   => $product['product_id'],
                        'master_id'    => $product['master_id'],
                        'name'         => $product['name'],
                        'model'        => $product['model'],
                        'option'       => $option_data,
                        'subscription' => $product['subscription'],
                        'download'     => $product['download'],
                        'quantity'     => $product['quantity'],
                        'subtract'     => $product['subtract'],
                        'price'        => $product['price'],
                        'total'        => $product['total'],
                        'tax'          => $this->tax->getTax($product['price'], $product['tax_class_id']),
                        'reward'       => $product['reward']
                    ];
			}
			$order_data['vouchers'] = [];
			if (!empty(  $session['vouchers'])) {
				$order_data['vouchers'] =   $session['vouchers'];
			}

			if (isset(  $session['comment'])) {
				$order_data['comment'] =   $session['comment'];
			} else {
				$order_data['comment'] = '';
			}
			$total_data = [
				'totals' => $totals,
				'taxes'  => $taxes,
				'total'  => $total
			];
			$order_data = array_merge($order_data, $total_data);
			$order_data['affiliate_id'] = 0;
			$order_data['commission'] = 0;
			$order_data['marketing_id'] = 0;
			$order_data['tracking'] = '';
			if ($this->config->get('config_affiliate_status') && isset(  $session['tracking'])) {
				$subtotal = $this->cart->getSubTotal();
				$this->load->model('account/affiliate');
				$affiliate_info = $this->model_account_affiliate->getAffiliateByTracking(  $session['tracking']);
				if ($affiliate_info) {
					$order_data['affiliate_id'] = $affiliate_info['customer_id'];
					$order_data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
					$order_data['tracking'] =   $session['tracking'];
				}
			}
			$order_data['language_id'] = $this->config->get('config_language_id');
			$order_data['language_code'] = $this->config->get('config_language');
			$order_data['currency_id'] = $this->currency->getId(  $session['currency']);
			$order_data['currency_code'] =   $session['currency'];
			$order_data['currency_value'] = $this->currency->getValue(  $session['currency']);
			$order_data['ip'] = $this->request->server['REMOTE_ADDR'];
			if (!empty($this->request->server['HTTP_X_FORWARDED_FOR'])) {
				$order_data['forwarded_ip'] = $this->request->server['HTTP_X_FORWARDED_FOR'];
			} elseif (!empty($this->request->server['HTTP_CLIENT_IP'])) {
				$order_data['forwarded_ip'] = $this->request->server['HTTP_CLIENT_IP'];
			} else {
				$order_data['forwarded_ip'] = '';
			}
			if (isset($this->request->server['HTTP_USER_AGENT'])) {
				$order_data['user_agent'] = $this->request->server['HTTP_USER_AGENT'];
			} else {
				$order_data['user_agent'] = '';
			}
			if (isset($this->request->server['HTTP_ACCEPT_LANGUAGE'])) {
				$order_data['accept_language'] = $this->request->server['HTTP_ACCEPT_LANGUAGE'];
			} else {
				$order_data['accept_language'] = '';
			}
			$this->load->model('checkout/order');
			$this->model_checkout_order->addOrder($order_data);
			$this->model_checkout_order->addHistory($order_id, '2');
            $this->cart->clear();
            unset($session['order_id']);
            unset($session['payment_address']);
            unset($session['payment_method']);
            unset($session['payment_methods']);
            unset($session['shipping_address']);
            unset($session['shipping_method']);
            unset($session['shipping_methods']);
            unset($session['comment']);
            unset($session['coupon']);
            unset($session['reward']);
            unset($session['voucher']);
            unset($session['vouchers']);
            $cloud = $this->config->get('payment_infrapay_cloud_key');
            $json['redirect'] = str_replace('//pay.php', '/pay.php', str_replace('admin.php', '', $this->config->get('payment_infrapay_url')) . '/pay.php?checkout_id=custom-oc&price=' . $total . '&currency=' . $order_data['currency_code'] . '&external-reference=' . bxc_oc_encryption($order_id) . ($order_data['language_code'] ? '&lang=' . substr($order_data['language_code'], 0, 2) : '') . '&redirect=' . urlencode($this->url->link('checkout/success', 'language=' . $order_data['language_code'], true)) . ($cloud ? '&cloud=' . trim($cloud) : ''));
			$this->response->addHeader('Content-Type: application/json');
            $this->response->setOutput(json_encode($json));
        }
	}

    public function callback(): void {
		$output = false;
        $response = json_decode(file_get_contents('php://input'), true);
		if (isset($response['transaction'])) {
			$transaction = $response['transaction'];
			$order_id = bxc_oc_encryption($transaction['external_reference'], false);
			if ($order_id && is_numeric($order_id)) {
				$this->load->model('checkout/order');
                $order = $this->model_checkout_order->getOrder($order_id);
                if (trim($response['key']) !== trim($this->config->get('payment_infrapay_webhook_key'))) $output = ['error' => 'Invalid webhook key.'];
                else if (floatval($order['total']) !== floatval($transaction['amount_fiat'])) $output = ['error' => 'Invalid amount.'];
                else {
                    $this->model_checkout_order->addHistory($order_id, 5, 'infrapay transaction ID: ' . $transaction['id']);
                    $output = ['success' => true];
                }
            } else {
                $output = ['error' => 'Invalid external_reference value: ' . $transaction['external_reference']];
            }
        }
		if (isset($output['error'])) bxc_oc_log($output['error']);
        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($output));
    }
}

function bxc_oc_encryption($string, $encrypt = true) {
    $output = false;
    $encrypt_method = 'AES-256-CBC';
	$password = DB_PASSWORD ? DB_PASSWORD : '12345678';
    $secret_key = $password . DB_USERNAME;
    $key = hash('sha256', $secret_key);
    $iv = substr(hash('sha256', $password), 0, 16);
    if ($encrypt) {
        $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
        $output = base64_encode($output);
        if (substr($output, -1) == '=') $output = substr($output, 0, -1);
    } else {
        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
    }
    return $output;
}

function bxc_oc_log($message) {
    $log = new \Opencart\System\Library\Log('infrapay.log');
    $log->write($message);
}